/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.tree;

import java.util.List;
import nl.cloudfarming.client.isobus.model.*;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle.Messages;

/**
 * <p>Creates Aggregate nodes for Isobus taskdata.xml nodes</p>
 *
 * <p>Can handle IsoBus types only! -----------------------------
 * Handles: </br>
 * {@link CommentAllocation}</br>
 * {@link Connection}</br>
 * {@link DeviceAllocation}</br>
 * {@link DeviceElement}</br>
 * {@link DeviceProcessData}</br>
 * {@link DeviceProperty}</br>
 * {@link DeviceValuePresentation}</br>
 * {@link DeviceObjectReference}</br>
 * {@link DataLog}</br>
 * {@link DataLogValue}</br>
 * {@link ProductAllocation}</br>
 * {@link IsobusTime}</br>
 * {@link IsobusTreatmentZone}</br>
 * {@link IsobusTask}</br>
 * {@link IsobusLineString}</br>
 * {@link IsobusPoint}</br>
 * {@link CropVariety}</br>
 * {@link WorkerAllocation}</br>
 *
 *
 * Trying to create an aggregate node for any other type will result in an
 * UnsupportedOperationException!
 * </p>
 *
 * @author Gerben Feenstra
 */
@Messages({
    "isobus project taskdata node name task=Task ({0})",
    "isobus project taskdata node name farm=Farm ({0})",
    "isobus project taskdata node name commentallocation=Comment allocation",
    "isobus project taskdata node name connection=Connection",
    "isobus project taskdata node name datalog=Data log",
    "isobus project taskdata node name datalog value=Data log value ({0})",
    "isobus project taskdata node name device allocation=Device allocation",
    "isobus project taskdata node name productallocation=Product allocation",
    "isobus project taskdata node name time=Time ({0} - {1})",
    "isobus project taskdata node name treatmentzone=TreatmentZone ({0})",
    "isobus project taskdata node name worker={0} - {1}",
    "isobus project taskdata node name workerallocation=Worker allocation ({0})",
    "isobus project taskdata node name device process data=Process data ({0})",
    "isobus project taskdata node name device value presentation=Device value presentation ({0})",
    "isobus project taskdata node name device property=Device property ({0})",
    "isobus project taskdata node name device object reference=DeviceObject reference ({0})"})
public class AggregateNodeChildFactory<I> extends ChildFactory<Node> {

    // Task
    private static final String TASK_NODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_task.png";
    private static final String COMMENT_ALLOCATION_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_comment_allocation.png";
    private static final String CONNECTION_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_connection.png";
    private static final String DEVICE_ALLOCATION_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_device_allocation.png";
    private static final String DATALOG_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_datalog.png";
    private static final String DATALOG_VALUE_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_datalog.png";
    private static final String PRODUCT_ALLOCATION_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_product_allocation.png";
    private static final String TIME_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_time.png";
    private static final String TREATMENTZONE_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_treatmentzone.png";
    private static final String WORKERALLOCATION_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_workerallocation.png";
    private static final String DEVICE_PROCESS_DATA_NODE_ICON = "";
    private static final String DEVICE_VALUE_PRESENTATION_NODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_value_presentation.png";
    private static final String DEVICEPROPERTY_NODE_ICON = "";
    private static final String DEVICEOBJECTREFERENCE_NODE_ICON = "";
    private List<I> isobusObjects;

    public AggregateNodeChildFactory(List<I> isobusObjects) {
        this.isobusObjects = isobusObjects;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {
        if (isobusObjects != null && isobusObjects.size() > 0) {

            // We are sure that all objects in the list are of the same type, 
            // Checking the first object to get access to the right for-loop is much less costly
            // then doing an instanceof for each object in the list. (List of Time objects can contain 150 objects for instance).
            I checkingInstance = isobusObjects.get(0);

            // Task Children
            if (checkingInstance instanceof CommentAllocation) {
                for (I isobusObject : isobusObjects) {
                    CommentAllocation commentAllocation = (CommentAllocation) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_commentallocation();
                    Children commentAllocationChildren = Children.create(new CommentAllocationChildFactory(commentAllocation), true);
                    toPopulate.add(new IsobusCommonNode(commentAllocationChildren, commentAllocation, COMMENT_ALLOCATION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof Connection) {
                for (I isobusObject : isobusObjects) {
                    Connection connection = (Connection) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_connection();
                    Children connectionChildren = Children.create(new ConnectionChildFactory(connection), true);
                    toPopulate.add(new IsobusCommonNode(connectionChildren, connection, CONNECTION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DeviceAllocation) {
                for (I isobusObject : isobusObjects) {
                    DeviceAllocation deviceAllocation = (DeviceAllocation) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_device_allocation();
                    Children deviceAllocationChildren = Children.create(new DeviceAllocationChildFactory(deviceAllocation), true);
                    toPopulate.add(new IsobusCommonNode(deviceAllocationChildren, deviceAllocation, DEVICE_ALLOCATION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DeviceElement) {
                for (I isobusObject : isobusObjects) {
                    DeviceElement deviceElement = (DeviceElement) isobusObject;
                    toPopulate.add(new DeviceElementNode(deviceElement));
                }
            } else if (checkingInstance instanceof DeviceProcessData) {
                for (I isobusObject : isobusObjects) {
                    DeviceProcessData deviceProcessData = (DeviceProcessData) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_device_process_data(deviceProcessData.getDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceProcessData, DEVICE_PROCESS_DATA_NODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DeviceProperty) {
                for (I isobusObject : isobusObjects) {
                    DeviceProperty deviceProperty = (DeviceProperty) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_device_property(deviceProperty.getDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceProperty, DEVICEPROPERTY_NODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DeviceValuePresentation) {
                for (I isobusObject : isobusObjects) {
                    DeviceValuePresentation deviceValuePresentation = (DeviceValuePresentation) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_device_value_presentation(deviceValuePresentation.getUnitDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceValuePresentation, DEVICE_VALUE_PRESENTATION_NODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DeviceObjectReference) {
                for (I isobusObject : isobusObjects) {
                    DeviceObjectReference deviceObjectReference = (DeviceObjectReference) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_device_object_reference(deviceObjectReference.getId());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceObjectReference, DEVICEOBJECTREFERENCE_NODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DataLog) {
                for (I isobusObject : isobusObjects) {
                    DataLog dataLog = (DataLog) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_datalog();
                    Children dataLogChildren = Children.create(new DataLogChildFactory(dataLog), true);
                    toPopulate.add(new IsobusCommonNode(dataLogChildren, dataLog, DATALOG_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DataLogValue) {
                for (I isobusObject : isobusObjects) {
                    DataLogValue dataLogValue = (DataLogValue) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_datalog_value(dataLogValue.getProcessDataValue());
                    Children dataLogChildren = Children.create(new DataLogValueChildFactory(dataLogValue), true);
                    toPopulate.add(new IsobusCommonNode(dataLogChildren, dataLogValue, DATALOG_VALUE_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof ProductAllocation) {
                for (I isobusObject : isobusObjects) {
                    ProductAllocation productAllocation = (ProductAllocation) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_productallocation();
                    Children productAllocationChildren = Children.create(new ProductAllocationChildFactory(productAllocation), true);
                    toPopulate.add(new IsobusCommonNode(productAllocationChildren, productAllocation, PRODUCT_ALLOCATION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof IsobusTime) {
                for (I isobusObject : isobusObjects) {
                    IsobusTime time = (IsobusTime) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_time(time.getTimeStart(), time.getTimeStop());
                    Children timeChildren = Children.create(new TimeChildFactory(time), true);
                    toPopulate.add(new IsobusCommonNode(timeChildren, time, TIME_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof IsobusTreatmentZone) {
                for (I isobusObject : isobusObjects) {
                    IsobusTreatmentZone treatmentZone = (IsobusTreatmentZone) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_treatmentzone(treatmentZone.getTreatmentZoneDesignator());
                    Children isobusTreatmentZoneChildren = Children.create(new TreatmentZoneChildFactory(treatmentZone), true);
                    toPopulate.add(new IsobusCommonNode(isobusTreatmentZoneChildren, treatmentZone, TREATMENTZONE_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof IsobusTask) {
                for (I isobusObject : isobusObjects) {
                    IsobusTask isobusTask = (IsobusTask) isobusObject;
                    String displayName = Bundle.isobus_project_taskdata_node_name_task(isobusTask.getTaskDesignator());
                    Children taskChildren = Children.create(new TaskChildFactory(isobusTask), true);
                    toPopulate.add(new IsobusCommonNode(taskChildren, isobusTask, TASK_NODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof IsobusLineString) {
                for (I isobusObject : isobusObjects) {
                    IsobusLineString lineString = (IsobusLineString) isobusObject;
                    toPopulate.add(new LineStringNode(lineString));
                }
            } else if (checkingInstance instanceof IsobusPoint) {
                for (I isobusObject : isobusObjects) {
                    IsobusPoint isobusPoint = (IsobusPoint) isobusObject;
                    toPopulate.add(new PointNode(isobusPoint));
                }
            } else if (checkingInstance instanceof CropVariety) {
                for (I isobusObject : isobusObjects) {
                    CropVariety cropVariety = (CropVariety) isobusObject;
                    toPopulate.add(new CropVarietyNode(cropVariety));
                }
            } else if (checkingInstance instanceof WorkerAllocation) {
                for (I isobusObject : isobusObjects) {
                    WorkerAllocation workerAllocation = (WorkerAllocation) isobusObject;
                    StringBuilder displayNameBuilder = new StringBuilder();
                    if (workerAllocation.getWorkerIdRef() != null) {
                        IsobusWorker worker = (IsobusWorker) workerAllocation.getWorkerIdRef();
                        if (worker.getWorkerDesignator() != null) {
                            displayNameBuilder.append(worker.getWorkerDesignator());
                            displayNameBuilder.append(" - ");
                        }
                    }
                    AllocationTime allocationTime = workerAllocation.getAllocationStamp();
                    if (allocationTime != null) {
                        if (allocationTime.getStartTime() != null) {
                            displayNameBuilder.append(allocationTime.getStartTime());
                        }
                        if (allocationTime.getEndTime() != null) {
                            displayNameBuilder.append(" - ");
                            displayNameBuilder.append(allocationTime.getEndTime());
                        }
                    }
                    String displayName = Bundle.isobus_project_taskdata_node_name_workerallocation(displayNameBuilder);
                    Children workerAllocationChildren = Children.create(new WorkerAllocationChildFactory(workerAllocation), true);
                    toPopulate.add(new IsobusCommonNode(workerAllocationChildren, workerAllocation, WORKERALLOCATION_CHILDNODE_ICON, displayName));
                }
            } else {
                throw new UnsupportedOperationException("Trying to create an node for an unsupported type: " + checkingInstance.getClass());
            }
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(Node key) {
        return key;
    }
}
