/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.tree;

import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.List;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.isobus.model.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle.Messages;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;

/**
 * A node for an Isobus PartField
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@Messages({
    "isobus project partfield node name=PartField ({0})"
})
public class PartFieldNode extends AbstractNode {

    private static final String PARTFIELD_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_partfield.png";
    private static final String PARTFIELD_LAYER_NAME = "IsobusPartfield_layer";

    /**
     * Create a PartFieldNode and specify a layer for this node in the lookup
     *
     * @param isobusPartField the isoBusPartField this node has to represent.
     * @param displayName Name to display for this node
     */
    public PartFieldNode(IsobusPartField isobusPartField) {
        super(Children.create(new PartFieldChildFactory(isobusPartField), true), Lookups.fixed(new SimpleLayer(PARTFIELD_LAYER_NAME), new PartFieldGeographical(isobusPartField)));
        this.setDisplayName(Bundle.isobus_project_partfield_node_name(isobusPartField.getDesignator()));
        this.setIconBaseWithExtension(PARTFIELD_CHILDNODE_ICON);
    }

    @Override
    public Transferable drag() throws IOException {
        return clipboardCopy();
    }

    @Override
    public Transferable clipboardCopy() throws IOException {
        return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
    }

    // TODO: Specialized help page for isobus field-nodes?
    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.isobus.project.explorer");
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    /**
     * Creates Children for Partfield-nodes.
     *
     * Partfields can have a collection of Polygons, LineStrings or Points.
     * PartFields have a reference to {@link Customer}, {@link IsobusFarm}, {@link CropType}, {@link CropVariety}
     * and other sub partfields
     *
     * @author Gerben Feenstra
     */
    @Messages({
        "isobus project taskdata node name croptype=Crop type ({0})"
    })
    static class PartFieldChildFactory extends ChildFactory<Node> {

        private static final String CROPTYPE_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_croptype_aggregate.png";
        private final IsobusPartField partField;

        public PartFieldChildFactory(IsobusPartField partField) {
            this.partField = partField;
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {

            //Customer (CTR)
            if (partField.getCustomerIdRef() != null) {
                toPopulate.add(new CustomerNode((Customer) partField.getCustomerIdRef()));
            }

            //Farm (FRM)
            if (partField.getFarmIdRef() != null) {
                toPopulate.add(new FarmNode((IsobusFarm) partField.getFarmIdRef()));
            }

            //CropType (CTP)
            if (partField.getCropTypeIdRef() != null) {
                CropType cropType = (CropType) partField.getCropTypeIdRef();
                String displayName = Bundle.isobus_project_taskdata_node_name_croptype(cropType.getName());
                toPopulate.add(new IsobusCommonNode(Children.create(new AggregateNodeChildFactory<>(cropType.getCropVarieties()), true), cropType, CROPTYPE_CHILDNODE_ICON, displayName));
            }

            //CropVariety (CVT)
            if (partField.getCropVarietyIdRef() != null) {
                toPopulate.add(new CropVarietyNode((CropVariety) partField.getCropVarietyIdRef()));
            }

            //PartField (PFD)
            if (partField.getFieldIdRef() != null) {
                toPopulate.add(new PartFieldNode((IsobusPartField) partField.getFieldIdRef()));
            }

            // LineString (LSG)
            for (IsobusLineString lineString : partField.getLSG()) {
                toPopulate.add(new LineStringNode(lineString));
            }

            // Polygon (PLN)
            for (IsobusPolygon polygon : partField.getPLN()) {
                toPopulate.add(new PolygonNode(polygon));
            }

            // Point (PNT)
            for (IsobusPoint point : partField.getPNT()) {
                toPopulate.add(new PointNode(point));
            }
            return true;
        }

        @Override
        protected Node createNodeForKey(Node node) {
            return node;
        }
    }
}
